/* Do not modify this file.  */
/* It is created automatically by the Makefile.  */
#include <algorithm>

#undef INIT_GDB_FILE
#define INIT_GDB_FILE(NAME) extern void _initialize_ ## NAME ();
INIT_GDB_FILE (loongarch_tdep)
INIT_GDB_FILE (loongarch_linux_tdep)
INIT_GDB_FILE (linux_tdep)
INIT_GDB_FILE (svr4_solib)
INIT_GDB_FILE (ser_hardwire)
INIT_GDB_FILE (ser_pipe)
INIT_GDB_FILE (ser_tcp)
INIT_GDB_FILE (ser_socket)
INIT_GDB_FILE (mdebugread)
INIT_GDB_FILE (cooked_index)
INIT_GDB_FILE (dwarf2_entry)
INIT_GDB_FILE (tailcall_frame)
INIT_GDB_FILE (dwarf2_frame)
INIT_GDB_FILE (index_cache)
INIT_GDB_FILE (dwarf_index_write)
INIT_GDB_FILE (dwarf2loc)
INIT_GDB_FILE (dwarf2_read)
INIT_GDB_FILE (read_gdb_index)
INIT_GDB_FILE (tui)
INIT_GDB_FILE (tui_disasm)
INIT_GDB_FILE (tui_hooks)
INIT_GDB_FILE (tui_interp)
INIT_GDB_FILE (tui_layout)
INIT_GDB_FILE (tui_regs)
INIT_GDB_FILE (tui_stack)
INIT_GDB_FILE (tui_win)
INIT_GDB_FILE (py_breakpoint)
INIT_GDB_FILE (py_connection)
INIT_GDB_FILE (py_interp)
INIT_GDB_FILE (py_micmd)
INIT_GDB_FILE (py_unwind)
INIT_GDB_FILE (python)
INIT_GDB_FILE (guile)
INIT_GDB_FILE (compile)
INIT_GDB_FILE (compile_cplus_types)
INIT_GDB_FILE (cp_name_parser)
INIT_GDB_FILE (ada_language)
INIT_GDB_FILE (tasks)
INIT_GDB_FILE (addrmap)
INIT_GDB_FILE (agent)
INIT_GDB_FILE (annotate)
INIT_GDB_FILE (gdbarch_utils)
INIT_GDB_FILE (auto_load)
INIT_GDB_FILE (auxv)
INIT_GDB_FILE (ax_gdb)
INIT_GDB_FILE (block)
INIT_GDB_FILE (break_catch_exec)
INIT_GDB_FILE (break_catch_fork)
INIT_GDB_FILE (break_catch_load)
INIT_GDB_FILE (break_catch_sig)
INIT_GDB_FILE (break_catch_syscall)
INIT_GDB_FILE (break_catch_throw)
INIT_GDB_FILE (break_cond_parse)
INIT_GDB_FILE (breakpoint)
INIT_GDB_FILE (btrace)
INIT_GDB_FILE (charset)
INIT_GDB_FILE (complaints)
INIT_GDB_FILE (completer)
INIT_GDB_FILE (copying)
INIT_GDB_FILE (core)
INIT_GDB_FILE (corelow)
INIT_GDB_FILE (cp_abi)
INIT_GDB_FILE (cp_namespace)
INIT_GDB_FILE (cp_support)
INIT_GDB_FILE (cp_valprint)
INIT_GDB_FILE (dcache)
INIT_GDB_FILE (debuginfod)
INIT_GDB_FILE (disasm)
INIT_GDB_FILE (displaced_stepping)
INIT_GDB_FILE (dummy_frame)
INIT_GDB_FILE (extract_store_integer)
INIT_GDB_FILE (event_top)
INIT_GDB_FILE (exec)
INIT_GDB_FILE (extension)
INIT_GDB_FILE (f_language)
INIT_GDB_FILE (f_valprint)
INIT_GDB_FILE (filesystem)
INIT_GDB_FILE (mem_search)
INIT_GDB_FILE (frame)
INIT_GDB_FILE (frame_unwind)
INIT_GDB_FILE (gcore)
INIT_GDB_FILE (gdb_demangle)
INIT_GDB_FILE (gdb_bfd)
INIT_GDB_FILE (gdbtypes)
INIT_GDB_FILE (gmp_utils)
INIT_GDB_FILE (gnu_v2_abi)
INIT_GDB_FILE (gnu_v3_abi)
INIT_GDB_FILE (infcall)
INIT_GDB_FILE (infcmd)
INIT_GDB_FILE (inflow)
INIT_GDB_FILE (infrun)
INIT_GDB_FILE (inline_frame)
INIT_GDB_FILE (interpreter)
INIT_GDB_FILE (jit)
INIT_GDB_FILE (language)
INIT_GDB_FILE (macrocmd)
INIT_GDB_FILE (macroscope)
INIT_GDB_FILE (maint_cmds)
INIT_GDB_FILE (maint_test_options)
INIT_GDB_FILE (maint_test_settings)
INIT_GDB_FILE (mem)
INIT_GDB_FILE (objc_language)
INIT_GDB_FILE (observer)
INIT_GDB_FILE (gdb_osabi)
INIT_GDB_FILE (osdata)
INIT_GDB_FILE (pascal_valprint)
INIT_GDB_FILE (parse)
INIT_GDB_FILE (printcmd)
INIT_GDB_FILE (probe)
INIT_GDB_FILE (producer)
INIT_GDB_FILE (psymtab)
INIT_GDB_FILE (record)
INIT_GDB_FILE (record_btrace)
INIT_GDB_FILE (record_full)
INIT_GDB_FILE (regcache)
INIT_GDB_FILE (regcache_dump)
INIT_GDB_FILE (reggroup)
INIT_GDB_FILE (remote)
INIT_GDB_FILE (notif)
INIT_GDB_FILE (reverse)
INIT_GDB_FILE (run_on_main_thread)
INIT_GDB_FILE (rust_exp)
INIT_GDB_FILE (serial)
INIT_GDB_FILE (step_skip)
INIT_GDB_FILE (solib)
INIT_GDB_FILE (source)
INIT_GDB_FILE (source_cache)
INIT_GDB_FILE (stabsread)
INIT_GDB_FILE (stack)
INIT_GDB_FILE (frame_reg)
INIT_GDB_FILE (symfile)
INIT_GDB_FILE (symfile_debug)
INIT_GDB_FILE (symmisc)
INIT_GDB_FILE (symtab)
INIT_GDB_FILE (target)
INIT_GDB_FILE (target_connection)
INIT_GDB_FILE (target_dcache)
INIT_GDB_FILE (target_descriptions)
INIT_GDB_FILE (thread)
INIT_GDB_FILE (top)
INIT_GDB_FILE (ctf)
INIT_GDB_FILE (tracefile)
INIT_GDB_FILE (tracefile_tfile)
INIT_GDB_FILE (tracepoint)
INIT_GDB_FILE (typeprint)
INIT_GDB_FILE (ui)
INIT_GDB_FILE (user_regs)
INIT_GDB_FILE (utils)
INIT_GDB_FILE (valops)
INIT_GDB_FILE (valprint)
INIT_GDB_FILE (values)
INIT_GDB_FILE (varobj)
INIT_GDB_FILE (xml_support)
INIT_GDB_FILE (cli_cmds)
INIT_GDB_FILE (cli_dump)
INIT_GDB_FILE (cli_interp)
INIT_GDB_FILE (cli_logging)
INIT_GDB_FILE (cli_script)
INIT_GDB_FILE (cli_style)
INIT_GDB_FILE (mi_cmd_env)
INIT_GDB_FILE (mi_cmds)
INIT_GDB_FILE (mi_interp)
INIT_GDB_FILE (mi_main)
INIT_GDB_FILE (dbxread)
INIT_GDB_FILE (mipsread)
INIT_GDB_FILE (coffread)
INIT_GDB_FILE (coff_pe_read)
INIT_GDB_FILE (xcoffread)
INIT_GDB_FILE (elfread)
INIT_GDB_FILE (stap_probe)
INIT_GDB_FILE (dtrace_probe)

void initialize_all_files ();
void
initialize_all_files ()
{
  typedef void initialize_file_ftype (void);

  std::vector<initialize_file_ftype *> functions =
    {
#undef INIT_GDB_FILE
#define INIT_GDB_FILE(NAME) _initialize_ ## NAME,
INIT_GDB_FILE (loongarch_tdep)
INIT_GDB_FILE (loongarch_linux_tdep)
INIT_GDB_FILE (linux_tdep)
INIT_GDB_FILE (svr4_solib)
INIT_GDB_FILE (ser_hardwire)
INIT_GDB_FILE (ser_pipe)
INIT_GDB_FILE (ser_tcp)
INIT_GDB_FILE (ser_socket)
INIT_GDB_FILE (mdebugread)
INIT_GDB_FILE (cooked_index)
INIT_GDB_FILE (dwarf2_entry)
INIT_GDB_FILE (tailcall_frame)
INIT_GDB_FILE (dwarf2_frame)
INIT_GDB_FILE (index_cache)
INIT_GDB_FILE (dwarf_index_write)
INIT_GDB_FILE (dwarf2loc)
INIT_GDB_FILE (dwarf2_read)
INIT_GDB_FILE (read_gdb_index)
INIT_GDB_FILE (tui)
INIT_GDB_FILE (tui_disasm)
INIT_GDB_FILE (tui_hooks)
INIT_GDB_FILE (tui_interp)
INIT_GDB_FILE (tui_layout)
INIT_GDB_FILE (tui_regs)
INIT_GDB_FILE (tui_stack)
INIT_GDB_FILE (tui_win)
INIT_GDB_FILE (py_breakpoint)
INIT_GDB_FILE (py_connection)
INIT_GDB_FILE (py_interp)
INIT_GDB_FILE (py_micmd)
INIT_GDB_FILE (py_unwind)
INIT_GDB_FILE (python)
INIT_GDB_FILE (guile)
INIT_GDB_FILE (compile)
INIT_GDB_FILE (compile_cplus_types)
INIT_GDB_FILE (cp_name_parser)
INIT_GDB_FILE (ada_language)
INIT_GDB_FILE (tasks)
INIT_GDB_FILE (addrmap)
INIT_GDB_FILE (agent)
INIT_GDB_FILE (annotate)
INIT_GDB_FILE (gdbarch_utils)
INIT_GDB_FILE (auto_load)
INIT_GDB_FILE (auxv)
INIT_GDB_FILE (ax_gdb)
INIT_GDB_FILE (block)
INIT_GDB_FILE (break_catch_exec)
INIT_GDB_FILE (break_catch_fork)
INIT_GDB_FILE (break_catch_load)
INIT_GDB_FILE (break_catch_sig)
INIT_GDB_FILE (break_catch_syscall)
INIT_GDB_FILE (break_catch_throw)
INIT_GDB_FILE (break_cond_parse)
INIT_GDB_FILE (breakpoint)
INIT_GDB_FILE (btrace)
INIT_GDB_FILE (charset)
INIT_GDB_FILE (complaints)
INIT_GDB_FILE (completer)
INIT_GDB_FILE (copying)
INIT_GDB_FILE (core)
INIT_GDB_FILE (corelow)
INIT_GDB_FILE (cp_abi)
INIT_GDB_FILE (cp_namespace)
INIT_GDB_FILE (cp_support)
INIT_GDB_FILE (cp_valprint)
INIT_GDB_FILE (dcache)
INIT_GDB_FILE (debuginfod)
INIT_GDB_FILE (disasm)
INIT_GDB_FILE (displaced_stepping)
INIT_GDB_FILE (dummy_frame)
INIT_GDB_FILE (extract_store_integer)
INIT_GDB_FILE (event_top)
INIT_GDB_FILE (exec)
INIT_GDB_FILE (extension)
INIT_GDB_FILE (f_language)
INIT_GDB_FILE (f_valprint)
INIT_GDB_FILE (filesystem)
INIT_GDB_FILE (mem_search)
INIT_GDB_FILE (frame)
INIT_GDB_FILE (frame_unwind)
INIT_GDB_FILE (gcore)
INIT_GDB_FILE (gdb_demangle)
INIT_GDB_FILE (gdb_bfd)
INIT_GDB_FILE (gdbtypes)
INIT_GDB_FILE (gmp_utils)
INIT_GDB_FILE (gnu_v2_abi)
INIT_GDB_FILE (gnu_v3_abi)
INIT_GDB_FILE (infcall)
INIT_GDB_FILE (infcmd)
INIT_GDB_FILE (inflow)
INIT_GDB_FILE (infrun)
INIT_GDB_FILE (inline_frame)
INIT_GDB_FILE (interpreter)
INIT_GDB_FILE (jit)
INIT_GDB_FILE (language)
INIT_GDB_FILE (macrocmd)
INIT_GDB_FILE (macroscope)
INIT_GDB_FILE (maint_cmds)
INIT_GDB_FILE (maint_test_options)
INIT_GDB_FILE (maint_test_settings)
INIT_GDB_FILE (mem)
INIT_GDB_FILE (objc_language)
INIT_GDB_FILE (observer)
INIT_GDB_FILE (gdb_osabi)
INIT_GDB_FILE (osdata)
INIT_GDB_FILE (pascal_valprint)
INIT_GDB_FILE (parse)
INIT_GDB_FILE (printcmd)
INIT_GDB_FILE (probe)
INIT_GDB_FILE (producer)
INIT_GDB_FILE (psymtab)
INIT_GDB_FILE (record)
INIT_GDB_FILE (record_btrace)
INIT_GDB_FILE (record_full)
INIT_GDB_FILE (regcache)
INIT_GDB_FILE (regcache_dump)
INIT_GDB_FILE (reggroup)
INIT_GDB_FILE (remote)
INIT_GDB_FILE (notif)
INIT_GDB_FILE (reverse)
INIT_GDB_FILE (run_on_main_thread)
INIT_GDB_FILE (rust_exp)
INIT_GDB_FILE (serial)
INIT_GDB_FILE (step_skip)
INIT_GDB_FILE (solib)
INIT_GDB_FILE (source)
INIT_GDB_FILE (source_cache)
INIT_GDB_FILE (stabsread)
INIT_GDB_FILE (stack)
INIT_GDB_FILE (frame_reg)
INIT_GDB_FILE (symfile)
INIT_GDB_FILE (symfile_debug)
INIT_GDB_FILE (symmisc)
INIT_GDB_FILE (symtab)
INIT_GDB_FILE (target)
INIT_GDB_FILE (target_connection)
INIT_GDB_FILE (target_dcache)
INIT_GDB_FILE (target_descriptions)
INIT_GDB_FILE (thread)
INIT_GDB_FILE (top)
INIT_GDB_FILE (ctf)
INIT_GDB_FILE (tracefile)
INIT_GDB_FILE (tracefile_tfile)
INIT_GDB_FILE (tracepoint)
INIT_GDB_FILE (typeprint)
INIT_GDB_FILE (ui)
INIT_GDB_FILE (user_regs)
INIT_GDB_FILE (utils)
INIT_GDB_FILE (valops)
INIT_GDB_FILE (valprint)
INIT_GDB_FILE (values)
INIT_GDB_FILE (varobj)
INIT_GDB_FILE (xml_support)
INIT_GDB_FILE (cli_cmds)
INIT_GDB_FILE (cli_dump)
INIT_GDB_FILE (cli_interp)
INIT_GDB_FILE (cli_logging)
INIT_GDB_FILE (cli_script)
INIT_GDB_FILE (cli_style)
INIT_GDB_FILE (mi_cmd_env)
INIT_GDB_FILE (mi_cmds)
INIT_GDB_FILE (mi_interp)
INIT_GDB_FILE (mi_main)
INIT_GDB_FILE (dbxread)
INIT_GDB_FILE (mipsread)
INIT_GDB_FILE (coffread)
INIT_GDB_FILE (coff_pe_read)
INIT_GDB_FILE (xcoffread)
INIT_GDB_FILE (elfread)
INIT_GDB_FILE (stap_probe)
INIT_GDB_FILE (dtrace_probe)
    };

  /* If GDB_REVERSE_INIT_FUNCTIONS is set (any value), reverse the
     order in which initialization functions are called.  This is
     used by the testsuite.  */
  if (getenv ("GDB_REVERSE_INIT_FUNCTIONS") != nullptr)
    std::reverse (functions.begin (), functions.end ());

  for (initialize_file_ftype *function : functions)
    function ();
}
